@@ -93,7 +93,7 @@ struct NautilusIconCanvasItemDetails {
 	guint rendered_is_highlighted_for_drop : 1;
 	guint rendered_is_prelit : 1;
 	
-	gboolean is_renaming;
+	guint is_renaming : 1;
 
 	PangoLayout *editable_text_layout;
 	PangoLayout *additional_text_layout;
@@ -133,7 +133,7 @@ typedef struct {
 	GList *emblem;
 } EmblemLayout;
 
-static  gboolean antialias_selection_rectangle = TRUE;
+#define ANTIALIAS_SELECTION_RECTANGLE TRUE
 
 static int click_policy_auto_value;
 
@@ -170,6 +170,7 @@ static void     draw_label_layout                    (NautilusIconCanvasItem
 						      GdkDrawable                   *drawable,
 						      PangoLayout                   *layout,
 						      gboolean                       highlight,
+						      GdkColor                      *label_color,
 						      int                            x,
 						      int                            y,
 						      GdkGC                         *gc);
@@ -558,8 +559,6 @@ nautilus_icon_canvas_item_update_bounds (NautilusIconCanvasItem *item)
 	/* queue a redraw. */
 	eel_gnome_canvas_request_redraw_rectangle
 		(GNOME_CANVAS_ITEM (item)->canvas, before);
-	eel_gnome_canvas_item_request_redraw
-		(GNOME_CANVAS_ITEM (item));
 }
 
 /* Update handler for the icon canvas item. */
@@ -570,7 +569,10 @@ nautilus_icon_canvas_item_update (GnomeCanvasItem *item,
 				  int flags)
 {
 	nautilus_icon_canvas_item_update_bounds (NAUTILUS_ICON_CANVAS_ITEM (item));
-	eel_gnome_canvas_item_request_redraw (item);
+
+	eel_gnome_canvas_item_request_redraw
+		(GNOME_CANVAS_ITEM (item));
+
 	EEL_CALL_PARENT (GNOME_CANVAS_ITEM_CLASS, update,
 			 (item, affine, clip_path, flags));
 }
@@ -583,8 +585,92 @@ in_single_click_mode (void)
 	return click_policy_auto_value == NAUTILUS_CLICK_POLICY_SINGLE;
 }
 
-/* Keep these for a bit while we work on performance of draw_or_measure_label_text. */
+/* Multiplies each pixel in a pixbuf by the specified color */
+static void
+multiply_pixbuf_rgba (GdkPixbuf *pixbuf, guint rgba)
+{
+	guchar *pixels;
+	int r, g, b, a;
+	int width, height, rowstride;
+	gboolean has_alpha;
+	int x, y;
+	guchar *p;
 
+	g_return_if_fail (gdk_pixbuf_get_colorspace (pixbuf) == GDK_COLORSPACE_RGB);
+	g_return_if_fail (gdk_pixbuf_get_n_channels (pixbuf) == 3
+			  || gdk_pixbuf_get_n_channels (pixbuf) == 4);
+
+	r = EEL_RGBA_COLOR_GET_R (rgba);
+	g = EEL_RGBA_COLOR_GET_G (rgba);
+	b = EEL_RGBA_COLOR_GET_B (rgba);
+	a = EEL_RGBA_COLOR_GET_A (rgba);
+
+	width = gdk_pixbuf_get_width (pixbuf);
+	height = gdk_pixbuf_get_height (pixbuf);
+	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+	has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
+
+	pixels = gdk_pixbuf_get_pixels (pixbuf);
+
+	for (y = 0; y < height; y++) {
+		p = pixels;
+
+		for (x = 0; x < width; x++) {
+			p[0] = p[0] * r / 255;
+			p[1] = p[1] * g / 255;
+			p[2] = p[2] * b / 255;
+
+			if (has_alpha) {
+				p[3] = p[3] * a / 255;
+				p += 4;
+			} else
+				p += 3;
+		}
+
+		pixels += rowstride;
+	}
+}
+
+static void
+draw_frame (NautilusIconCanvasItem *item,
+	    GdkDrawable *drawable,
+	    guint color,
+	    int x, 
+	    int y,
+	    int width,
+	    int height)
+{
+	GdkPixbuf *selection_pixbuf;
+	NautilusIconContainer *container;
+
+	container = NAUTILUS_ICON_CONTAINER (GNOME_CANVAS_ITEM (item)->canvas);
+
+	if (ANTIALIAS_SELECTION_RECTANGLE) {
+		selection_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+						   TRUE,
+						   8,
+						   width,
+						   height);
+		eel_gdk_pixbuf_fill_rectangle_with_color (selection_pixbuf,
+							  eel_gdk_pixbuf_whole_pixbuf,
+							  0xffffffff);
+		clear_rounded_corners (selection_pixbuf,
+				       container->details->highlight_frame,
+				       5);
+		multiply_pixbuf_rgba (selection_pixbuf,
+				      color);
+		
+		draw_pixbuf (selection_pixbuf, drawable, x, y);
+		g_object_unref (selection_pixbuf);
+	} else {
+		gdk_draw_rectangle
+			(drawable, 
+			 GTK_WIDGET (container)->style->black_gc, TRUE,
+			 x, y, width, height);
+	}
+}
+
+/* Keep these for a bit while we work on performance of draw_or_measure_label_text. */
 /*
   #define PERFORMANCE_TEST_DRAW_DISABLE
   #define PERFORMANCE_TEST_MEASURE_DISABLE
@@ -598,14 +684,14 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 {
 	NautilusIconCanvasItemDetails *details;
 	NautilusIconContainer *container;
-	guint width_so_far, height_so_far;
+	guint editable_height, editable_width;
+	guint additional_height, additional_width;
 	GnomeCanvasItem *canvas_item;
-	GdkPixbuf *selection_pixbuf;
-	PangoLayout *layout;
+	PangoLayout *editable_layout;
+	PangoLayout *additional_layout;
 	GdkColor *label_color;
-	int layout_width, layout_height;
 	int icon_width;
-	gboolean have_editable, have_additional, needs_highlight;
+	gboolean have_editable, have_additional, needs_highlight, needs_frame;
 	int max_text_width, box_left;
 	GdkGC *gc;
 	
@@ -649,134 +735,131 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 
 	canvas_item = GNOME_CANVAS_ITEM (item);
 	if (drawable != NULL) {
-		gc = gdk_gc_new (canvas_item->canvas->layout.bin_window);
 		icon_width = details->pixbuf == NULL ? 0 : gdk_pixbuf_get_width (details->pixbuf);
 	}
 	
-	width_so_far = 0;
-	height_so_far = 0;
+	editable_width = 0;
+	editable_height = 0;
+	additional_width = 0;
+	additional_height = 0;
 
 	max_text_width = floor (nautilus_icon_canvas_item_get_max_text_width (item));
 
 	container = NAUTILUS_ICON_CONTAINER (GNOME_CANVAS_ITEM (item)->canvas);	
-				
-	/* if the icon is highlighted, do some set-up */
-	if (needs_highlight && drawable != NULL && !details->is_renaming &&
-	    details->text_width > 0 && details->text_height > 0) {
-		if (antialias_selection_rectangle) {
-			selection_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-							   TRUE,
-							   8,
-							   details->text_width,
-							   details->text_height);
-			eel_gdk_pixbuf_fill_rectangle_with_color (selection_pixbuf,
-								  eel_gdk_pixbuf_whole_pixbuf,
-								  container->details->highlight_color_rgba);
-			clear_rounded_corners (selection_pixbuf, container->details->highlight_frame, 5);
-			draw_pixbuf (selection_pixbuf, drawable, 
-				     icon_left + (icon_width - details->text_width) / 2,
-				     icon_bottom);
-			g_object_unref (selection_pixbuf);
-		} else {
-			gdk_gc_set_rgb_fg_color (gc, &container->details->highlight_color);
-			
-			gdk_draw_rectangle
-				(drawable, GTK_WIDGET (GNOME_CANVAS_ITEM (item)->canvas)->style->black_gc, TRUE,
-				 icon_left + (icon_width - details->text_width) / 2,
-				 icon_bottom,
-				 details->text_width, details->text_height);
-		}
-		
-	}	
+	editable_layout = NULL;
+	additional_layout = NULL;
 
 	if (have_editable) {
-		layout = get_label_layout (&details->editable_text_layout, item, details->editable_text);
-
-		if (drawable != NULL) {
-			label_color = nautilus_icon_container_get_label_color
-				(NAUTILUS_ICON_CONTAINER (canvas_item->canvas),
-				 TRUE, needs_highlight);
-
-			gdk_gc_set_rgb_fg_color (gc, label_color);
-			
-			draw_label_layout (item, drawable,
-					   layout, needs_highlight,
-					   icon_left + (icon_width - max_text_width) / 2,
-					   icon_bottom, gc);
-		}
+		editable_layout = get_label_layout (&details->editable_text_layout, item, details->editable_text);
 		
-		pango_layout_get_pixel_size (layout, &layout_width, &layout_height);
-		
-		width_so_far = MAX (width_so_far, (guint) layout_width);
-		height_so_far += layout_height + LABEL_LINE_SPACING;
-		
-		g_object_unref (layout);
+		pango_layout_get_pixel_size (editable_layout, 
+					     &editable_width,
+					     &editable_height);
 	}
-	
+
 	if (have_additional) {
-		layout = get_label_layout (&details->additional_text_layout, item, details->additional_text);
-
-		if (drawable != NULL) {
-			label_color = nautilus_icon_container_get_label_color
-				(NAUTILUS_ICON_CONTAINER (canvas_item->canvas),
-				 FALSE, needs_highlight);
-
-			gdk_gc_set_rgb_fg_color (gc, label_color);
-			
-			draw_label_layout (item, drawable,
-					   layout, needs_highlight,
-					   icon_left + (icon_width - max_text_width) / 2,
-					   icon_bottom + height_so_far, gc);
-		}
-		
-		pango_layout_get_pixel_size (layout, &layout_width, &layout_height);
-
-		width_so_far = MAX (width_so_far, (guint) layout_width);
-		height_so_far += layout_height + LABEL_LINE_SPACING;
+		additional_layout = get_label_layout (&details->additional_text_layout, item, details->additional_text);
 
+		pango_layout_get_pixel_size (additional_layout, 
+					     &additional_width,
+					     &additional_height);
 	}
-	
-	if (antialias_selection_rectangle) {
+
+	details->text_width = MAX (editable_width, additional_width);
+	details->text_height = editable_height + LABEL_LINE_SPACING + additional_height;
+
+	if (ANTIALIAS_SELECTION_RECTANGLE) {
 		/* add some extra space for highlighting even when we don't highlight so things won't move */
-		height_so_far += 2; /* extra slop for nicer highlighting */	
-		width_so_far += 8;  /* account for emboldening, plus extra to make it look nicer */
+		details->text_height += 2; /* extra slop for nicer highlighting */	
+		details->text_width += 8;  /* extra to make it look nicer */
 	} else {
 		/* add slop used for highlighting, even if we're not highlighting now */
-		width_so_far += 4;
+		details->text_width += 4;
 	}
 
-	if (drawable != NULL) {
-		/* Current calculations should match what we measured before drawing.
-		 * This assumes that we will always make a separate call to measure
-		 * before the call to draw. We might later decide to use this function
-		 * differently and change these asserts.
-		 */
-#if (defined PERFORMANCE_TEST_MEASURE_DISABLE || defined PERFORMANCE_TEST_DRAW_DISABLE)
-		g_assert ((int) height_so_far == details->text_height);
-		g_assert ((int) width_so_far == details->text_width);
-#endif
-		box_left = icon_left + (icon_width - details->text_width) / 2;
-
-		if (item->details->is_highlighted_as_keyboard_focus) {
-			gtk_paint_focus (GTK_WIDGET (GNOME_CANVAS_ITEM (item)->canvas)->style,
-					 drawable,
-					 needs_highlight ? GTK_STATE_SELECTED : GTK_STATE_NORMAL,
-					 NULL,
-					 GTK_WIDGET (GNOME_CANVAS_ITEM (item)->canvas),
-					 "icon-container",
-					 box_left,
-					 icon_bottom,
-					 details->text_width,
-					 details->text_height);
+	/* if measuring, we are done */
+	if (!drawable) {
+		if (editable_layout) {
+			g_object_unref (editable_layout);
 		}
-	} else {
-		/* If measuring, remember the width & height. */
-		details->text_width = width_so_far;
-		details->text_height = height_so_far;
+
+		if (additional_layout) {
+			g_object_unref (additional_layout);
+		}
+
+		return;
+	}
+	
+	/* if the icon is highlighted, do some set-up */
+	if (needs_highlight && !details->is_renaming &&
+	    details->text_width > 0 && details->text_height > 0) {
+		draw_frame (item,
+			    drawable,
+			    container->details->highlight_color_rgba,
+			    icon_left + (icon_width - details->text_width) / 2,
+			    icon_bottom,
+			    details->text_width, 
+			    details->text_height);
 	}
 
-	if (gc != NULL) {
-		g_object_unref (gc);
+	if (have_editable) {
+		gtk_widget_style_get (GTK_WIDGET (container),
+				      "frame_text", &needs_frame,
+				      NULL);
+		if (needs_frame && !needs_highlight && details->text_width > 0 && details->text_height > 0) {
+			draw_frame (item, 
+				    drawable,
+				    eel_gdk_color_to_rgb (&GTK_WIDGET (container)->style->base[GTK_STATE_NORMAL]),
+				    icon_left + (icon_width - details->text_width) / 2,
+				    icon_bottom,
+				    details->text_width, 
+				    details->text_height);	    
+		}
+		
+		gc = nautilus_icon_container_get_label_color_and_gc
+			(NAUTILUS_ICON_CONTAINER (canvas_item->canvas),
+			 &label_color, TRUE, needs_highlight);
+		
+		draw_label_layout (item, drawable,
+				   editable_layout, needs_highlight,
+				   label_color,
+				   icon_left + (icon_width - max_text_width) / 2,
+				   icon_bottom, gc);
+	}
+
+	if (have_additional) {
+		gc = nautilus_icon_container_get_label_color_and_gc
+			(NAUTILUS_ICON_CONTAINER (canvas_item->canvas),
+			 &label_color, FALSE, needs_highlight);
+		
+		draw_label_layout (item, drawable,
+				   additional_layout, needs_highlight,
+				   label_color,
+				   icon_left + (icon_width - max_text_width) / 2,
+				   icon_bottom + editable_height + LABEL_LINE_SPACING, gc);
+	}
+
+	box_left = icon_left + (icon_width - details->text_width) / 2;
+
+	if (item->details->is_highlighted_as_keyboard_focus) {
+		gtk_paint_focus (GTK_WIDGET (GNOME_CANVAS_ITEM (item)->canvas)->style,
+				 drawable,
+				 needs_highlight ? GTK_STATE_SELECTED : GTK_STATE_NORMAL,
+				 NULL,
+				 GTK_WIDGET (GNOME_CANVAS_ITEM (item)->canvas),
+				 "icon-container",
+				 box_left,
+				 icon_bottom,
+				 details->text_width,
+				 details->text_height);
+	}
+
+	if (editable_layout) {
+		g_object_unref (editable_layout);
+	}
+	
+	if (additional_layout) {
+		g_object_unref (additional_layout);
 	}
 }
 
@@ -1212,6 +1295,7 @@ draw_label_layout (NautilusIconCanvasItem *item,
 		   GdkDrawable *drawable,
 		   PangoLayout *layout,
 		   gboolean highlight,
+		   GdkColor *label_color,
 		   int x,
 		   int y,
 		   GdkGC *gc)
@@ -1224,24 +1308,17 @@ draw_label_layout (NautilusIconCanvasItem *item,
 		return;
 	}
 
-	if (!highlight || !antialias_selection_rectangle) {
-		gdk_draw_layout (drawable, gc,
-				 x, y,
-				 layout);
+	if (!highlight && (NAUTILUS_ICON_CONTAINER (GNOME_CANVAS_ITEM (item)->canvas)->details->use_drop_shadows)) {
+		/* draw a drop shadow */
+		eel_gdk_draw_layout_with_drop_shadow (drawable, gc,
+						      label_color,
+						      &GTK_WIDGET (GNOME_CANVAS_ITEM (item)->canvas)->style->black,
+						      x, y,
+						      layout);
 	} else {
-		/* draw a shadow in black */
-		gdk_draw_layout (drawable,
-				 GTK_WIDGET (GNOME_CANVAS_ITEM (item)->canvas)->style->black_gc,
-				 x + 2, y + 1,
-				 layout);
-		
-		/* draw smeared-wide text to "embolden" */
 		gdk_draw_layout (drawable, gc,
 				 x, y,
 				 layout);
-		gdk_draw_layout (drawable, gc,
-				 x+1, y,
-				 layout);
 	}
 }
 
@@ -1521,10 +1598,10 @@ nautilus_icon_canvas_item_bounds (GnomeCanvasItem *item,
         
 	/* Return the result. */
 	pixels_per_unit = item->canvas->pixels_per_unit;
-	*x1 = total_rect.x0 / pixels_per_unit;
-	*y1 = total_rect.y0 / pixels_per_unit;
-	*x2 = total_rect.x1 / pixels_per_unit;
-	*y2 = total_rect.y1 / pixels_per_unit;
+	*x1 = floor (total_rect.x0 / pixels_per_unit);
+	*y1 = floor (total_rect.y0 / pixels_per_unit);
+	*x2 = ceil (total_rect.x1 / pixels_per_unit) + 1;
+	*y2 = ceil (total_rect.y1 / pixels_per_unit) + 1;
 }
 
 /* Get the rectangle of the icon only, in world coordinates. */
@@ -1682,7 +1759,7 @@ nautilus_icon_canvas_item_set_renaming (NautilusIconCanvasItem *item, gboolean s
 	g_return_if_fail (NAUTILUS_IS_ICON_CANVAS_ITEM (item));
 	g_return_if_fail (state == FALSE || state == TRUE);
 
-	if (item->details->is_renaming == state) {
+	if (!item->details->is_renaming == !state) {
 		return;
 	}
 
